"""Context app builder."""

from typing import Callable, Dict
from pydantic import BaseSettings

from .fn import setup_logger, new_log
from .ext.api import API

from ._config import ServiceConfig
from ._sentry import init_sentry

log = new_log(__name__)


def _as_service(app: API, config: ServiceConfig, options: Dict) -> None:
    """Config service app."""
    for router in config.routers:
        app.include_router(router)


def _as_rpc(app: API, config: ServiceConfig, options: Dict) -> None:
    """Config rpc app."""
    app.routes.clear()


def build(
    service_config: ServiceConfig,
    settings: BaseSettings = None,
    callbacks: Dict[str, Callable] = None,
    prefix: str = '/',
    prepare: bool = True,
    disable_doc: bool = False,
    **options,
) -> API:
    """Build app."""
    init_kw = {
        'prefix': prefix,
        'debug': service_config.debug,
    }

    skip_doc_kw = {
        'openapi_url': None,
        'swagger_ui_parameters': {'defaultModelsExpandDepth': -1},
    }

    if prepare and disable_doc:
        init_kw.update(skip_doc_kw)

    app = API(**init_kw)

    if service_config.log_config.exists():
        setup_logger(service_config.log_config.as_posix(), debug=service_config.debug)
    else:
        log.warning(
            f'Log config not found: {service_config.log_config.as_posix()}',
        )

    init_sentry(service_config)

    if not prepare:
        # Skip any for possible pipeline using
        log.warning('App is not prepared by request')
        return app

    app.config = service_config
    app.title = service_config.service_name
    app.description = service_config.service_info

    if settings:
        app.settings = settings

    if callbacks:
        for func_name, method in callbacks.items():
            app.add_event_handler(func_name, method)

    configure = {True: _as_rpc, False: _as_service}
    configure[service_config.rpc_mode](app, service_config, options)

    for middleware in service_config.middleware:
        app.add_middleware(middleware)

    return app


__all__ = ('build', )
