"""Logging factory.

Just call setup_logger before use default logging as usual.

Check your log configuration before wasting time here.
Attention: Do not import any project modules or func here (!!!).
"""

import os
import sys
import logging
import json

from collections.abc import Mapping
from typing import List
import traceback
from pathlib import Path

from . import logger
from . import term


def formatter_from_event_type(event_type: int):
    """Map formatter to event."""
    log_styles = {
        logging.NOTSET: term.StyleDefault,
        logging.DEBUG: term.StyleDebug,
        logging.INFO: term.StyleInfo,
        logging.WARNING: term.StyleWarning,
        logging.CRITICAL: term.StyleError,
        logging.ERROR: term.StyleError,
        logging.FATAL: term.StyleError,
    }
    try:
        return log_styles[event_type]
    except (AttributeError, KeyError, IndexError):
        return term.StyleDefault


class Formatter(logger.JSONFormatter):
    """Default log formatter."""

    @classmethod
    def clear_trace(cls, lines: List[str]) -> List[str]:
        """Clear exc trace."""
        trace_buff = []
        for line in lines:
            buff = line.strip('')
            buff = buff.strip('\"')
            buff = buff.replace('\n', '')
            buff = buff.replace("\"/", '/')
            buff = buff.replace('\"', '')
            if buff.startswith('None'):
                continue
            trace_buff.append(buff)
        return trace_buff

    def format(self, record: logger.LogRecord):
        """Format."""
        super().format(record)

        message = {
            field_name: logger.formatter_value(record, field_value)
            for field_name, field_value in self.fields.items()
        }

        if isinstance(record.msg, Mapping):
            message.update(record.msg)
        else:
            message[self.message_field_name] = super().formatMessage(record)

        message.update(logger.log_extra_attrs(record))
        message['msg'] = record.msg or record.message

        if record.name == 'sqlalchemy.engine.Engine':
            _no_sql = ('\n', '\t')
            for _spec in _no_sql:
                message['msg'] = str(message['msg']).replace(_spec, ' ')

        caller = Path(record.pathname)

        message['path'] = '../{parent}/{name}/{script}'.format(
            parent=caller.parent.parent.name,
            name=caller.parent.name,
            script=caller.name,
        )

        _debug = 'true' in str(os.environ.get('DEBUG', '')).lower()

        if record.levelname in ('CRITICAL', 'ERROR'):
            exc_type, exc_value, exc_traceback = sys.exc_info()
            if exc_type:
                message['exc_type'] = exc_type.__name__
            if exc_value and str(exc_value) != '':
                message['exc_value'] = exc_value

            exc_lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
            exc_lines = self.clear_trace(exc_lines)
            if exc_lines:
                message['msg'] = exc_lines

        if len(message) == 1 and self.message_field_name in message:
            return super().formatMessage(record)

        _fmt_style = formatter_from_event_type(record.levelno)

        _colorize = 'true' in str(os.environ.get('LOG_HIGHLIGHT', '')).lower()

        msg = json.dumps(
            message,
            default=logger.default_converter,
            indent=4 if _colorize else 1,  # single line for elk
            ensure_ascii=False,
        )

        if _colorize:
            # for development only
            return term.highlight(
                msg,
                lexer=term.JsonLexer(),
                formatter=term.Terminal256Formatter(style=_fmt_style),
            )

        return msg


class RPCFormatter(Formatter):

    """RPC Log formatter."""

    pass


__all__ = ('Formatter', 'RPCFormatter')
